package de.cscherr.mcpht.mixin.client;

import de.cscherr.mcpht.MCPHTClient;
import de.cscherr.mcpht.config.MCPHTConfig;
import net.minecraft.network.ClientConnection;
import org.slf4j.Logger;
import net.minecraft.network.PacketCallbacks;
import net.minecraft.network.listener.PacketListener;
import net.minecraft.network.packet.Packet;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.jetbrains.annotations.Nullable;

@Mixin(ClientConnection.class)
public class NetworkPacketLogMixin {
	@Unique
	private static final Logger LOGGER = MCPHTClient.LOGGER;
	/**
	 * Inject to the method that sends packets. Must not be static for some reason
	 *
	 * @param packet the Network packet to be sent
	 * @param callbacks magical fabric mixin stuff
	 * @param ci magical fabric mixin stuff
	 */
	@Inject(method = "sendImmediately", at = @At("HEAD"))
	public void logSentPacket(Packet<?> packet, @Nullable PacketCallbacks callbacks, CallbackInfo ci) {
		if (MCPHTClient.CONFIG.networkLogging.TX) {
			String information = null;
			switch (MCPHTClient.CONFIG.networkLogging.verbosity) {
				case NAME ->  {
					information = packet.getClass().getSimpleName();
					break;
				}
				case ALL -> {
					information = String.format("%s:\n" +
							"	%s\n" +
							"ALL INFO", packet.getClass().getSimpleName(), packet.toString());
					break;
				}
			}
			LOGGER.info(String.format("Sent Package: %s",
					information
			));
		}
	}

	/**
	 * Inject to the method that receives and handles packets.
	 * Must be static for some reason
	 *
	 * @param packet the Network packet to be sent
	 * @param listener i dont know what this does
	 * @param ci magical fabric mixin stuff
	 */
	@Inject(method = "handlePacket", at = @At("HEAD"))
	private static void logReceivedPacket(Packet<?> packet, PacketListener listener, CallbackInfo ci) {
		if (MCPHTClient.CONFIG.networkLogging.RX) {
			LOGGER.info(String.format("Received Package: %s",
					packet.getClass().getSimpleName()
			));
		}
	}

}
